#coding: utf-8

class SelectionRule
  include Mongoid::Document
  include Mongoid::Timestamps
  field :published, :type => DateTime
  field :content
  field :language
  field :author
  field :category
  embedded_in :source

  # Aplica totes els filtres de la regla de manera seqüencial
  def apply_all
    selected_items = self.source.items
    [:published, :content, :language, :author, :category].each do |rule|
      selected_items = selected_items & self.apply(rule)
    end
    selected_items
  end

  # Aplica un filtre concret que es passa com a paràmetre
  def apply(rule)
    return self.source.items if self.send(rule).blank?
    case rule
      when :published
        # Aplicant filtre segons temporalitat
        self.source.items.where(:published.gt => self.published).entries
      when :content
        # Aplicant filtre segons paraules al contingut
        self.source.items.where({ content: /#{self.content}/i }).entries
      when :language
        # Aplicant filtre segons idioma
        self.source.items.where(:language => self.language).entries
      when :author
        # Aplicant filtre segons autoria
        self.source.items.where({ author: /#{self.author}/i }).entries
      when :category
        # Aplicant filtre segons pertenença a una categoria
        selected_items = Array.new
        self.source.items.each do |item|
          selected_items << item if item.categories.split(',').include?(self.category)
        end
        # Retornem els items filtrats
        selected_items
    end
  end

end
