#coding: utf-8

require 'open-uri'

module FeedParser

  @feed = nil

  # Mètode envoltori que detecta el format de la font i l'interpreta
  def parse
    # Obrim el fitxer que contè el feed
    @feed = Nokogiri::XML(open(self.url))
    if is_rss_feed?
      parse_rss
    elsif is_atom_feed?
      parse_atom
    end
  end

  # Ens diu si es tracta d'una font d'informació en format Atom
  def is_atom_feed?
    @feed.xpath('/xmlns:feed').any?
  end

  # Ens diu si es tracta d'una font d'informació en format RSS 2.0
  def is_rss_feed?
    @feed.xpath('/rss').any?
  end


  # Mètodes privats
  private

  # Interpreta un font en format Atom
  def parse_atom
    # Omplim els camps que identifiquen la font d'informació
    self.title = @feed.xpath('/xmlns:feed/xmlns:title').text
    self.description = @feed.xpath('/xmlns:feed/xmlns:subtitle').text
    self.url = @feed.xpath('/xmlns:feed/xmlns:link[@rel="self"]/@href').text
    self.updated = @feed.xpath('/xmlns:feed/xmlns:updated').text
    
    # Guardem cadascun dels items de la font
    self.items.destroy_all
    @feed.xpath('/xmlns:feed/xmlns:entry').each do |feed_entry|
      self.items.new(
        title: feed_entry.xpath('xmlns:title').text,
        url: feed_entry.xpath('xmlns:link[@rel="alternate"]/@href').first.text,
        author: feed_entry.xpath('xmlns:author').text.strip,
        content: feed_entry.xpath('xmlns:content').text.strip,
        published: feed_entry.xpath('xmlns:published').text,
        item_id: feed_entry.xpath('xmlns:id').text,
        categories: feed_entry.xpath('xmlns:category/@term').to_a.join(',')
      )
    end
  end

  # Interpreta un font en format RSS 2.0
  def parse_rss
    # Omplim els camps que identifiquen la font d'informació
    self.title = @feed.xpath('/rss/channel/title').text
    self.description = @feed.xpath('/rss/channel/description').text
    self.updated = @feed.xpath('/rss/channel/lastBuildDate').text
    self.language = @feed.xpath('/rss/channel/language').text
    
    # Guardem cadascun dels items de la font
    self.items.destroy_all
    @feed.xpath('/rss/channel/item').each do |feed_entry|
      # Cerquem el contingut al camp 'content:encoded', però si no existeix agafem el camp 'description'
      content = begin
                  if feed_entry.xpath('content:encoded').blank?
                    feed_entry.xpath('description').text.strip
                  else
                    feed_entry.xpath('content:encoded').text.strip
                  end
                rescue Nokogiri::XML::XPath::SyntaxError
                  feed_entry.xpath('description').text.strip
                end
      self.items.new(
        title: feed_entry.xpath('title').text,
        url: feed_entry.xpath('link').text,
        author: feed_entry.xpath('author').text,
        content: content,
        published: feed_entry.xpath('pubDate').text,
        item_id: feed_entry.xpath('guid').text,
        categories: feed_entry.xpath('category').to_a.join(',')
      )
    end

  end

end
