# coding:UTF-8

require 'zip/zipfilesystem'

module OpendocumentExporter
  NAMESPACES = {'xmlns:office' => 'urn:oasis:names:tc:opendocument:xmlns:office:1.0',
                             'xmlns:text' => 'urn:oasis:names:tc:opendocument:xmlns:text:1.0',
                             'xmlns:meta' => 'urn:oasis:names:tc:opendocument:xmlns:meta:1.0',
                             'xmlns:dc' => 'http://purl.org/dc/elements/1.1/',
                             'xmlns:xlink' => 'http://www.w3.org/1999/xlink'}

  def export
    file_name = "exports/#{self.title}-#{Time.now.to_i}.odt"
    Zip::ZipFile.open(file_name, Zip::ZipFile::CREATE) do |zipfile|
      zipfile.file.open("content.xml", "w") { |f| f.write self.generate_opendocument_content }
      zipfile.file.open("meta.xml", "w") { |f| f.write self.generate_opendocument_metadata }
      zipfile.file.open("mimetype", "w") { |f| f.write 'application/vnd.oasis.opendocument.text' }
      zipfile.dir.mkdir("META-INF")
      zipfile.file.open("META-INF/manifest.xml", "w") { |f| f.write self.generate_opendocument_manifest }
    end
    file_name
  end

  def generate_opendocument_manifest
    builder = Nokogiri::XML::Builder.new do |xml|
      xml.manifest('xmlns:manifest' => 'urn:oasis:names:tc:opendocument:xmlns:manifest:1.0') {
        xml.parent.namespace = xml.parent.namespace_definitions.find{ |ns| ns.prefix=='manifest' }
        xml['manifest'].send(:'file-entry',
          :'manifest:media-type' => 'application/vnd.oasis.opendocument.text',
          :'manifest:version' => '1.2',
          :'manifest:full-path' => '/'
          ) {}
        ['content.xml', 'meta.xml'].each do |path|
          xml['manifest'].send(:'file-entry',
            :'manifest:media-type' => 'text/xml',
            :'manifest:full-path' => path
            ) {}
        end
      }
    end
    builder.to_xml
  end

  def generate_opendocument_metadata
    builder = Nokogiri::XML::Builder.new do |xml|
      xml.send(:'document-meta', OpendocumentExporter::NAMESPACES) {
        xml.parent.namespace = xml.parent.namespace_definitions.find{ |ns| ns.prefix=='office' }
        xml['office'].meta {
          xml['meta'].send(:'creation-date', Time.now.iso8601)
          xml['meta'].generator 'PFC Compiladors 2011'
          xml['dc'].language 'ca-ES'
          xml['dc'].date Time.now.iso8601
          xml['dc'].creator 'Abraham Sánchez'
        }
      }
    end
    builder.to_xml

  end

  def generate_opendocument_content
    builder = Nokogiri::XML::Builder.new do |xml|
      xml.send(:'document-content', OpendocumentExporter::NAMESPACES) {
        xml.parent.namespace = xml.parent.namespace_definitions.find{|ns|ns.prefix=='office'}
        xml['office'].send(:'automatic-styles') {}
        xml['office'].body {
          xml['office'].text_ {
            xml['text'].h self.title
            xml['text'].p helpers.strip_tags(self.description)
            self.items.each do |item|
              xml['text'].p {}
              xml['text'].p {
                xml['text'].a('xlink:type' => 'simple', 'xlink:href' => item.url) {
                  xml['text'].span('text:style-name' => 'T1') { xml.text item.title }
                }
              }
              xml['text'].p item.author
              xml['text'].p helpers.strip_tags(item.content)
            end
          }
        }
      }
    end
    builder.to_xml
  end

  def helpers
    ActionController::Base.helpers
  end

end
