require 'spec_helper'

# This spec was generated by rspec-rails when you ran the scaffold generator.
# It demonstrates how one might use RSpec to specify the controller code that
# was generated by Rails when you ran the scaffold generator.
#
# It assumes that the implementation code is generated by the rails scaffold
# generator.  If you are using any extension libraries to generate different
# controller code, this generated spec may or may not pass.
#
# It only uses APIs available in rails and/or rspec-rails.  There are a number
# of tools you can use to make these specs even more expressive, but we're
# sticking to rails and rspec-rails APIs to keep things simple and stable.
#
# Compared to earlier versions of this generator, there is very limited use of
# stubs and message expectations in this spec.  Stubs are only used when there
# is no simpler way to get a handle on the object needed for the example.
# Message expectations are only used when there is no simpler way to specify
# that an instance is receiving a specific message.

describe SourcesController do

  before(:each) do
    @source = Factory.create(:source)
  end

  describe "GET index" do
    it "assigns all sources as @sources" do
      get :index
      assigns(:sources).entries.should eq([@source])
    end
  end

  describe "GET show" do
    it "assigns the requested source as @source" do      
      get :show, :id => @source.id
      assigns(:source).should eq(@source)
    end
  end

  describe "GET new" do
    it "assigns a new source as @source" do
      get :new
      assigns(:source).should be_a_new(Source)
    end
  end

  describe "GET edit" do
    it "assigns the requested source as @source" do      
      get :edit, :id => @source.id
      assigns(:source).should eq(@source)
    end
  end
  
  describe "GET export" do
    it "exports the requested source" do      
      @source.selection_rule = Factory.create(:selection_rule)        
      @source.items << Factory.create(:item)
      get :export, :id => @source.id
      response.should redirect_to(sources_url)
    end
  end
  
  describe "GET refresh" do
    it "refreshes the requested source" do
      @source.selection_rule = Factory.create(:selection_rule)        
      get :refresh, :id => @source.id
       response.should redirect_to(sources_url)
    end
  end
  
  describe "POST create" do
    describe "with valid params" do
      it "creates a new Source" do
        expect {
          post :create, :source => {:url => @source.url}
        }.to change(Source, :count).by(1)
      end

      it "assigns a newly created source as @source" do
        post :create, :source => @source.attributes
        assigns(:source).should be_a(Source)
        assigns(:source).should be_persisted
      end

      it "redirects to the created source" do
        post :create, :source => @source.attributes
        response.should redirect_to(Source.last)
      end
    end 
  end

  describe "PUT update" do
    describe "with valid params" do
      it "updates the requested source" do
        # Assuming there are no other sources in the database, this
        # specifies that the Source created on the previous line
        # receives the :update_attributes message with whatever params are
        # submitted in the request.
        Source.any_instance.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => @source.id, :source => {'these' => 'params'}
      end

      it "assigns the requested source as @source" do
        @source.selection_rule = Factory.create(:selection_rule)        
        put :update, :id => @source.id, :source => @source.attributes
        assigns(:source).should eq(@source)
      end

      it "redirects to the source" do
        @source.selection_rule = Factory.create(:selection_rule)        
        put :update, :id => @source.id, :source => @source.attributes
        response.should redirect_to(@source)
      end
    end

    describe "with invalid params" do
      it "assigns the source as @source" do
        # Trigger the behavior that occurs when invalid params are submitted
        Source.any_instance.stub(:save).and_return(false)
        put :update, :id => @source.id, :source => {}
        assigns(:source).should eq(@source)
      end

      it "re-renders the 'edit' template" do
        # Trigger the behavior that occurs when invalid params are submitted
        Source.any_instance.stub(:save).and_return(false)
        put :update, :id => @source.id, :source => {}
        response.should render_template("edit")
      end
    end
  end

  describe "DELETE destroy" do
    it "destroys the requested source" do
      expect {
        delete :destroy, :id => @source.id
      }.to change(Source, :count).by(-1)
    end

    it "redirects to the sources list" do
      delete :destroy, :id => @source.id
      response.should redirect_to(sources_url)
    end
  end

end
